/*

Copyright (C) 2005, 2006 John L. Weatherwax

This file is part of Octave.

Octave is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Octave is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Octave; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   
   Written by:
   -- 
   John L. Weatherwax                2005-05-15
   
   email: wax@alum.mit.edu
   
   Please send comments and especially bug reports to the
   above email address.
   
*/

#include <octave/oct.h>
#include <octave/f77-fcn.h>

extern "C" {
  int F77_FUNC(pdecol,PDECOL) ( double&, double&, double&, double*, double&, \
				int&, int&, int&, int&, int&, int&, \
				double*, int* ); 
  int F77_FUNC(values,VALUES) ( double*, double*, double*, int&, int&, int&, int&, double* );
}

// Global pointer for user defined function required by pdecol.
static octave_function *pdecol_fcn;

// C++ octave API call of user function:
ColumnVector
pdecol_user_function (double t, double x, const ColumnVector& u, 
		      const ColumnVector & ux, const ColumnVector& uxx)
{
  ColumnVector retval;

  octave_value_list args;
  args(0) = t; args(1) = x; args(2) = u; args(3) = ux; args(4) = uxx; 

  if (pdecol_fcn)
    {
      octave_value_list tmp = pdecol_fcn->do_multi_index_op (1, args);

      if (error_state)
	{
	  gripe_user_supplied_eval ("pdecol");
	  return retval;
	}

      if (tmp.length () > 0 && tmp(0).is_defined ())
	{
	  retval = ColumnVector (tmp(0).vector_value ());

	  if (error_state || retval.length () == 0)
	    gripe_user_supplied_eval ("pdecol");
	}
      else
	gripe_user_supplied_eval ("pdecol");
    }

  return retval;
}


DEFUN_DLD(pdecolg, args, nargout, "pdecolg"){

  octave_value_list retval; 

  // Input argument checking: 
  int nargin=args.length(); 
  if( nargin != 11 ){
    error("pdecolg: Incorrect number of input arguments");
    return retval;
  }
  if( nargout != 1 ) {
    error( "pdecolg: Incorrect number of output arguments");
    return retval;
  }

  // Extract all input arguments: 
  // 

  // Extract all inputs needed for the FORTRAN call:
  //
  RowVector touts( args(0).vector_value() );
  double *toutsP = touts.fortran_vec();
  int ntouts = args(0).length()-1;
  double dt = args(1).double_value();
  RowVector xbkpt( args(2).vector_value() );
  double *xbkptP = xbkpt.fortran_vec();
  int npts = args(2).length();
  double eps = args(3).double_value();
  int kord = args(4).int_value();
  int ncc = args(5).int_value(); 
  int mf = args(6).int_value();
  int npde = args(7).int_value();
  // Create storage for work/iwork to pass into FORTRAN code:
  int nwork = args(8).int_value();
  OCTAVE_LOCAL_BUFFER( double, workP, nwork ); 
  int niwork = args(9).int_value();
  OCTAVE_LOCAL_BUFFER( int, iworkP, niwork );
  iworkP[0] = nwork; 
  iworkP[1] = niwork; 
  int nsctch = args(10).int_value(); 
  OCTAVE_LOCAL_BUFFER( double, sctchP, nsctch ); 

  // u declared as an "NDArray":
  NDArray uT( dim_vector(npde,npts,ntouts) );
  Array<int> dim(3,0);

  int NINT=npts-1; 
  int INDEX=1;
  for( int tcnt = 0; tcnt < ntouts; tcnt++ ){
    // CALL SUBROUTINE "PDECOL":
    double T0=touts(tcnt);
    double TOUT=touts(tcnt+1);
    F77_FUNC(pdecol,PDECOL)(T0,TOUT,dt,xbkptP,eps,NINT,kord,ncc,npde,mf,\
			    INDEX,workP,iworkP);
    // CALL SUBROUTINE "VALUES":
    int derivOrder = 0; 
    Matrix uTS( npde, npts );
    double *uTSP = uTS.fortran_vec(); 
    F77_FUNC(values,VALUES)(xbkptP,uTSP,sctchP,npde,npts,npts,derivOrder,workP);

    // Assign the location where this page of results goes: 
    dim(2)=tcnt; 
    uT = uT.concat(uTS,dim); 
  }
  // Store to pass out:
  retval.append(uT);
  return retval;
}
 
